@@ -28,7 +28,7 @@
     }
 }
 
-niftyreg <- function (source, target, targetMask = NULL, initAffine = NULL, scope = c("affine","rigid"), nLevels = 3, maxIterations = 5, useBlockPercentage = 50, finalInterpolation = 3, verbose = FALSE)
+niftyreg <- function (source, target, targetMask = NULL, initAffine = NULL, scope = c("affine","rigid"), nLevels = 3, maxIterations = 5, useBlockPercentage = 50, finalInterpolation = 3, verbose = FALSE, interpolationPrecision = NULL)
 {
     if (!require("oro.nifti"))
         report(OL$Error, "The \"oro.nifti\" package is required")
@@ -63,16 +63,24 @@ niftyreg <- function (source, target, targetMask = NULL, initAffine = NULL, scop
     
     scope <- match.arg(scope)
     
+    if (!is.null(interpolationPrecision))
+        interpolationPrecision <- match.arg(interpolationPrecision, c("source","single","double"))
+    else if (finalInterpolation == 0)
+        interpolationPrecision <- "source"
+    else
+        interpolationPrecision <- "single"
+    
     if (source@dim_[1] == target@dim_[1])
     {
-        returnValue <- .Call("reg_aladin", .fixTypes(source), .fixTypes(target), scope, as.integer(nLevels), as.integer(maxIterations), as.integer(useBlockPercentage), as.integer(finalInterpolation), .fixTypes(targetMask), initAffine, as.integer(verbose), PACKAGE="RNiftyReg")
-
+        returnValue <- .Call("reg_aladin", .fixTypes(source), .fixTypes(target), scope, interpolationPrecision, as.integer(nLevels), as.integer(maxIterations), as.integer(useBlockPercentage), as.integer(finalInterpolation), .fixTypes(targetMask), initAffine, as.integer(verbose), PACKAGE="RNiftyReg")
+        
         dim(returnValue[[1]]) <- dim(target)
         dim(returnValue[[2]]) <- c(4,4)
         attr(returnValue[[2]], "affineType") <- "niftyreg"
 
         resultImage <- as.nifti(returnValue[[1]], target)
         affine <- list(returnValue[[2]])
+        iterations <- list(returnValue[[3]])
     }
     else
     {
@@ -80,23 +88,25 @@ niftyreg <- function (source, target, targetMask = NULL, initAffine = NULL, scop
         finalDims <- c(dim(target), dim(source)[nSourceDims])
         nReps <- finalDims[length(finalDims)]
         finalArray <- array(0, dim=finalDims)
-        affine <- vector("list", nReps)
+        affine <- iterations <- vector("list", nReps)
+
         for (i in seq_len(nReps))
         {
             if (nSourceDims == 3)
             {
-                returnValue <- .Call("reg_aladin", .fixTypes(as.nifti(source[,,i],source)), .fixTypes(target), scope, as.integer(nLevels), as.integer(maxIterations), as.integer(useBlockPercentage), as.integer(finalInterpolation), .fixTypes(targetMask), initAffine, as.integer(verbose), PACKAGE="RNiftyReg")
+                returnValue <- .Call("reg_aladin", .fixTypes(as.nifti(source[,,i],source)), .fixTypes(target), scope, interpolationPrecision, as.integer(nLevels), as.integer(maxIterations), as.integer(useBlockPercentage), as.integer(finalInterpolation), .fixTypes(targetMask), initAffine, as.integer(verbose), PACKAGE="RNiftyReg")
                 finalArray[,,i] <- returnValue[[1]]
             }
             else if (nSourceDims == 4)
             {
-                returnValue <- .Call("reg_aladin", .fixTypes(as.nifti(source[,,,i],source)), .fixTypes(target), scope, as.integer(nLevels), as.integer(maxIterations), as.integer(useBlockPercentage), as.integer(finalInterpolation), .fixTypes(targetMask), initAffine, as.integer(verbose), PACKAGE="RNiftyReg")
+                returnValue <- .Call("reg_aladin", .fixTypes(as.nifti(source[,,,i],source)), .fixTypes(target), scope, interpolationPrecision, as.integer(nLevels), as.integer(maxIterations), as.integer(useBlockPercentage), as.integer(finalInterpolation), .fixTypes(targetMask), initAffine, as.integer(verbose), PACKAGE="RNiftyReg")
                 finalArray[,,,i] <- returnValue[[1]]
             }
             
             dim(returnValue[[2]]) <- c(4,4)
             attr(returnValue[[2]], "affineType") <- "niftyreg"
             affine[[i]] <- returnValue[[2]]
+            iterations[[i]] <- returnValue[[3]]
         }
         
         resultImage <- as.nifti(finalArray, target)
@@ -107,16 +117,18 @@ niftyreg <- function (source, target, targetMask = NULL, initAffine = NULL, scop
     resultImage@cal_max <- max(resultImage@.Data)
     resultImage@scl_slope <- source@scl_slope
     resultImage@scl_inter <- source@scl_inter
-    resultImage@datatype <- source@datatype
-    resultImage@bitpix <- as.numeric(source@bitpix)
     
-    result <- list(image=resultImage, affine=affine, scope=scope)
+    resultImage@datatype <- switch(interpolationPrecision, source=source@datatype, single=16L, double=64L)
+    resultImage@data_type <- convert.datatype(resultImage@datatype)
+    resultImage@bitpix <- switch(interpolationPrecision, source=as.numeric(source@bitpix), single=32, double=64)
+    
+    result <- list(image=resultImage, affine=affine, iterations=iterations, scope=scope)
     class(result) <- "niftyreg"
     
     return (result)
 }
 
-applyAffine <- function (affine, source, target, affineType = NULL, finalInterpolation = 3)
+applyAffine <- function (affine, source, target, affineType = NULL, finalInterpolation = 3, interpolationPrecision = NULL)
 {
     if (!is.matrix(affine) || !isTRUE(all.equal(dim(affine), c(4,4))))
         report(OL$Error, "Specified affine matrix is not valid")
@@ -130,5 +142,5 @@ applyAffine <- function (affine, source, target, affineType = NULL, finalInterpo
     else
         attr(affine, "affineType") <- affineType
     
-    return (niftyreg(source, target, targetMask=NULL, initAffine=affine, scope="affine", nLevels=0, finalInterpolation=finalInterpolation, verbose=FALSE))
+    return (niftyreg(source, target, targetMask=NULL, initAffine=affine, scope="affine", nLevels=0, finalInterpolation=finalInterpolation, interpolationPrecision=interpolationPrecision, verbose=FALSE))
 }
